# -*- coding: utf-8 -*-
# j2.py
#
# Copyright 2011-2019 Hiroshi Miura <miurahr@linux.com>

import re
import threading

from klepto.archives import file_archive

from .properties import Configurations
from .scripts import H2


class J2(object):
    _kanwa = None
    _itaiji = None

    _cl_table = ["", "aiueow", "aiueow", "aiueow", "aiueow", "aiueow", "aiueow", "aiueow",
                 "aiueow", "aiueow", "aiueow", "k", "g", "k", "g", "k", "g", "k", "g", "k",
                 "g", "s", "zj", "s", "zj", "s", "zj", "s", "zj", "s", "zj", "t", "d", "tc",
                 "d", "aiueokstchgzjfdbpw", "t", "d", "t", "d", "t", "d", "n", "n", "n", "n",
                 "n", "h", "b", "p", "h", "b", "p", "hf", "b", "p", "h", "b", "p", "h", "b",
                 "p", "m", "m", "m", "m", "m", "y", "y", "y", "y", "y", "y", "rl", "rl",
                 "rl", "rl", "rl", "wiueo", "wiueo", "wiueo", "wiueo", "w", "n", "v", "k",
                 "k", "", "", "", "", "", "", "", "", ""]

    def __init__(self, mode, method="Hepburn"):
        self._kanwa = Kanwa()
        self._itaiji = Itaiji()
        if mode == "H":
            self.convert = self.convert_H
        elif mode in ("a", "K"):
            self._hconv = H2(mode, method)
            self.convert = self.convert_nonH
        else:
            self.convert = self.convert_noop

    def isRegion(self, c):
        return 0x3400 <= ord(c[0]) < 0xe000 or 0xf900 <= ord(c[0]) < 0xfa2e

    def isCletter(self, l, c):
        if (0x3041 <= ord(c) <= 0x309f) and (l in self._cl_table[ord(c) - 0x3040]):  # ぁ:= u\3041
            return True
        return False

    def itaiji_conv(self, text):
        r = []
        for c in text:
            if self._itaiji.haskey(c):
                r.append(c)
        for c in r:
            text = re.sub(c, self._itaiji.lookup(c), text)
        return text

    def convert_H(self, text):
        max_len = 0
        Hstr = ""
        text = self._itaiji.convert(text)
        table = self._kanwa.load(text[0])
        if table is None:
            return "", 0
        for (k, v) in table.items():
            length = len(k)
            if len(text) >= length:
                if text.startswith(k):
                    for (yomi, tail) in v:
                        if tail == '':
                            if max_len < length:
                                Hstr = yomi
                                max_len = length
                        elif max_len < length + 1 and len(text) > length \
                                and self.isCletter(tail, text[length]):
                            Hstr = ''.join([yomi, text[length]])
                            max_len = length + 1
        return (Hstr, max_len)

    def convert_nonH(self, text):
        if not self.isRegion(text[0]):
            return "", 0

        (t, l1) = self.convert_H(text)
        if l1 <= 0:  # pragma: no cover
            return "", 0

        m = 0
        otext = ""

        while True:
            if m >= len(t):
                break
            (s, n) = self._hconv.convert(t[m:])
            if n <= 0:  # pragma: no cover
                m = m + 1
            else:
                m = m + n
                otext = otext + s

        return otext, l1

    def convert_noop(self, text):
        return text[0], 1


class Itaiji(object):

    # this class is Borg/Singleton
    _shared_state = {
        '_itaijidict': None,
        '_itaijidict_len': 0,
        '_lock': threading.Lock()
    }

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._itaijidict is None:
            with self._lock:
                if self._itaijidict is None:
                    itaijipath = Configurations().dictpath(Configurations().jisyo_itaiji)
                    self._itaijidict = file_archive(itaijipath, {}, serialized=True)
                    self._itaijidict.load()
                    self._itaijidict_len = self._itaijidict['_max_key_len_']

    def haskey(self, key):
        return key in self._itaijidict

    def lookup(self, key):
        return self._itaijidict[key]

    def convert(self, text):
        r = []
        for c in text:
            if self.haskey(c):
                r.append(c)
        for c in r:
            text = re.sub(c, self.lookup(c), text)
        return text


# This class is Borg/Singleton
# It provides same results becase lookup from a static dictionary.
# There is no state rather dictionary dbm.
class Kanwa(object):
    _shared_state = {
        '_lock': threading.Lock(),
        '_jisyo_table': None
    }

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._jisyo_table is None:
            with self._lock:
                if self._jisyo_table is None:
                    dictpath = Configurations().dictpath(Configurations().jisyo_kanwa)
                    self._jisyo_table = file_archive(dictpath, {}, serialized=True)
                    self._jisyo_table.load()

    def load(self, char):
        key = "%04x" % ord(char)
        return self._jisyo_table.get(key, None)



import os
import sys

import pytest


def pytest_addoption(parser):
    parser.addoption('--runenv', action='store', default="pytest", help='Specify tox or pytest')


@pytest.fixture(scope='session')
def runenv(request):
    return request.config.getoption('runenv')


@pytest.fixture(scope="session", autouse=True)
def dictionary_setup_fixture(runenv):
    root_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    if runenv == 'pytest':
        sys.path.insert(1, os.path.join(root_dir, 'src'))
        import kakasidict
        from pykakasi.properties import Configurations
        dpath = os.path.join(root_dir, 'build', 'lib', 'pykakasi', 'data')
        print("Generating kanwa dictionary in %s\n" % dpath)
        kanwa = kakasidict.Genkanwadict()
        kanwa.generate_dictionaries(dpath)
        Configurations().data_path = dpath
    elif runenv == 'tox':
        pass
    else:
        pass



# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011-2018 Hiroshi Miura <miurahr@linux.com>
#

__license__ = 'GPL 3'
__copyright__ = '2014-2019 Hiroshi Miura <miurahr@linux.com>'
__docformat__ = 'restructuredtext en'

'''
Flags:
  These flags are as same as KAKASI.

  p: List all possible readings. If there exist two or more
     possible readings, KAKASI shows them in braces {aaa,bbb}.(not implemented yet)
  s: Insert a separate character between words.
  f: Furigana mode. Shows the original kanji word with reading.
  c: Skip characters within word. ( default TAB CR LF BLANK )(not implemented yet)
  C: Capitalize Romaji word (with -Ja or -Jj option)
  U: Upcase romaji word (with -Ja or -Jj option)
  u: Unbufferd mode.(not implemented yet)
  w: wakatigaki mode. 'wakatigaki' is word segmentation for
     Japanese sentences.(implemented as wakati class)

'''

import six

from .exceptions import (InvalidFlagValueException, InvalidModeValueException,
                         UnknownOptionsException,
                         UnsupportedRomanRulesException)
from .kanji import J2
from .properties import Ch
from .scripts import A2, H2, K2, Sym2


class kakasi:

    _keys = ["J", "H", "K", "E", "a"]
    _values = ["a", "E", "H", "K"]
    _roman_vals = ["Hepburn", "Kunrei", "Passport"]
    _MAXLEN = 32
    _LONG_SYMBOL = [
        # 0x002D,  # -
        0x30FC,  # ー
        # 0x2010,  # ‐
        # 0x2011,  # ‑
        # 0x2013,  # –
        # 0x2014,  # —
        0x2015,  # ―
        0x2212,  # −
        0xFF70  # ｰ
    ]

    def __init__(self):
        self._conv = {}
        self._mode = {"J": None, "H": None, "K": None, "E": None, "a": None}
        self._furi = {"J": False, "H": False, "K": False, "E": False, "a": False}
        self._flag = {"p": False, "s": False, "f": False, "c": False, "C": False, "U": False,
                      "u": False, "t": True}
        self._option = {"r": "Hepburn"}
        self._separator = ' '

    def setMode(self, fr, to):
        if fr in self._keys:
            if to is None:
                self._mode[fr] = to
            elif to[0] in self._values:
                self._mode[fr] = to[0]
                if len(to) == 2 and to[1] == "F":
                    self._furi[fr] = True
            else:
                raise InvalidModeValueException("Invalid value for mode")
        elif fr in self._flag.keys():
            if to in [True, False]:
                self._flag[fr] = to
            else:
                raise InvalidFlagValueException("Invalid flag value")
        elif fr == "r":
            if to in self._roman_vals:
                self._option["r"] = to
            else:
                raise UnsupportedRomanRulesException("Unknown roman table name")
        elif fr == "S":
            self._separator = to
        else:
            raise UnknownOptionsException("Unhandled options")  # pragma: no cover

    def getConverter(self):
        self._conv["J"] = J2(self._mode["J"], method=self._option["r"])
        self._conv["H"] = H2(self._mode["H"], method=self._option["r"])
        self._conv["K"] = K2(self._mode["K"], method=self._option["r"])
        self._conv["E"] = Sym2(self._mode["E"])
        self._conv["a"] = A2(self._mode["a"])
        return self

    def do(self, itext):
        text = six.text_type(itext)

        otext = ""
        i = 0
        while True:
            if i >= len(text):
                break

            if self._conv["J"].isRegion(text[i]):
                mode = "J"
            elif self._conv["H"].isRegion(text[i]):
                mode = "H"
            elif self._conv["K"].isRegion(text[i]):
                mode = "K"
            elif self._conv["E"].isRegion(text[i]):
                mode = "E"
            elif self._conv["a"].isRegion(text[i]):
                mode = "a"
            else:
                mode = "o"

            if mode in ("J", "E"):
                w = min(i + self._MAXLEN, len(text))
                (t, l1) = self._conv[mode].convert(text[i:w])

                if l1 > 0:
                    orig = text[i:i + l1]
                    chunk = t
                    i += l1
                else:
                    orig = text[i:i + 1]
                    if self._flag["t"]:
                        chunk = orig
                    else:
                        chunk = "???"
                    i += 1

            elif mode in ("H", "K", "a"):
                orig = ''
                chunk = ''

                while i < len(text):

                    # 長音記号の場合は、直前の文字とreplaceする
                    if ord(text[i]) in self._LONG_SYMBOL:

                        if self._mode[mode] is not None:
                            orig += text[i]
                            chunk = chunk + chunk[-1]
                            i += 1
                        else:
                            orig += text[i]
                            chunk += text[i]
                            i += 1
                            break

                    elif self._conv[mode].isRegion(text[i]):
                        w = min(i + self._MAXLEN, len(text))
                        (t, l1) = self._conv[mode].convert(text[i:w])
                        if l1 > 0:
                            orig += text[i:i + l1]
                            chunk += t
                            i += l1
                        else:
                            orig = text[i:i + 1]
                            if self._flag["t"]:
                                chunk = orig
                            else:
                                chunk = "???"
                            i += 1
                            break

                    else:
                        # i += 1
                        break

            else:
                otext += text[i]
                i += 1
                continue

            if mode in ("J", "E"):
                if self._flag["U"]:
                    chunk = chunk.upper()
                elif self._flag["C"]:
                    chunk = chunk.capitalize()

            if mode in self._keys and self._furi[mode]:
                otext += orig + "[" + chunk + "]"
            else:
                otext += chunk

            # insert separator when option specified and it is not a last character and not an end mark
            if self._flag["s"] and otext[-len(self._separator):] != self._separator \
                    and i < len(text) and not (ord(text[i]) in Ch.endmark):
                otext += self._separator

        return otext


class wakati(kakasi):

    _jconv = None
    _separator = " "
    _state = True

    def __init__(self):
        self._jconv = J2("H")
        self._flag = {"f": False}

    def getConverter(self):
        return self

    def setMode(self, fr, to):
        if fr in self._flag.keys():
            if to in [True, False]:
                self._flag[fr] = to
            else:
                raise InvalidFlagValueException("Invalid flag value")
            raise UnknownOptionsException("Unhandled options")

    def do(self, text):

        if len(text) == 0:
            return ""

        otext = ''
        i = 0
        while True:
            if i >= len(text):
                break

            if self._jconv.isRegion(text[i]):
                _, ln = self._jconv.convert(text[i:])
                if ln <= 0:  # pragma: no cover
                    otext = otext + text[i]
                    i += 1
                    self._state = False
                elif (i + ln) < len(text):
                    if self._state:
                        otext = otext + text[i:i + ln] + self._separator
                    else:
                        otext = otext + self._separator + text[i:i + ln] + self._separator
                        self._state = True
                    i = i + ln
                else:
                    if self._state:
                        otext = otext + text[i:i + ln]
                    else:  # pragma: no cover
                        otext = otext + self._separator + text[i:i + ln]
                    break

            else:
                self._state = False
                otext = otext + text[i]
                i += 1

        return otext



# -*- coding: utf-8 -*-
# scripts.py
#
# Copyright 2011-2019 Hiroshi Miura <miurahr@linux.com>

from klepto.archives import file_archive
from six import unichr
from six.moves import xrange

from .exceptions import UnsupportedRomanRulesException
from .properties import Ch, Configurations, Convert_Tables


class H2 (object):

    _kanadict = None

    _diff = 0x30a1 - 0x3041  # KATAKANA LETTER A - HIRAGANA A

    def __init__(self, mode, method="Hepburn"):
        conf = Configurations()
        if mode == "a":
            if method == "Hepburn":
                self._kanadict = Jisyo(conf.jisyo_hepburn_hira)
            elif method == "Passport":
                self._kanadict = Jisyo(conf.jisyo_passport_hira)
            elif method == "Kunrei":
                self._kanadict = Jisyo(conf.jisyo_kunrei_hira)
            else:
                raise UnsupportedRomanRulesException("Unsupported roman rule")

            self.convert = self.convert_a
        elif mode == "K":
            self.convert = self.convert_K
        else:
            self.convert = self.convert_noop

    def isRegion(self, char):
        return ((0x3040 < ord(char[0]) and ord(char[0]) < 0x3097))

    def convert_a(self, text):
        Hstr = ""
        max_len = -1
        r = min(self._kanadict.maxkeylen(), len(text))
        for x in xrange(1, r + 1):
            if self._kanadict.haskey(text[:x]):
                if max_len < x:
                    max_len = x
                    Hstr = self._kanadict.lookup(text[:x])
        return (Hstr, max_len)

    def convert_K(self, text):
        Hstr = ""
        max_len = 0
        r = len(text)
        for x in xrange(r):
            if self.isRegion(text[x]):
                Hstr = Hstr + unichr(ord(text[x]) + self._diff)
                max_len += 1
            else:  # pragma: no cover
                break
        return (Hstr, max_len)

    def convert_noop(self, text):
        return (text[0], 1)


class K2 (object):

    _kanadict = None

    _diff = 0x30a1 - 0x3041  # KATAKANA LETTER A - HIRAGANA A

    def __init__(self, mode, method="Hepburn"):
        conf = Configurations()
        if mode == "a":
            if method == "Hepburn":
                self._kanadict = Jisyo(conf.jisyo_hepburn)
            elif method == "Passport":
                self._kanadict = Jisyo(conf.jisyo_passport)
            elif method == "Kunrei":
                self._kanadict = Jisyo(conf.jisyo_kunrei)
            else:
                raise UnsupportedRomanRulesException("Unsupported roman rule")  # pragma: no cover

            self.convert = self.convert_a
        elif mode == "H":
            self.convert = self.convert_h
        else:
            self.convert = self.convert_noop

    def isRegion(self, char):
        return 0x30a0 < ord(char[0]) < 0x30fd

    def convert_a(self, text):
        Hstr = ""
        max_len = -1
        r = min(self._kanadict.maxkeylen(), len(text))
        for x in xrange(1, r + 1):
            if self._kanadict.haskey(text[:x]):
                if max_len < x:
                    max_len = x
                    Hstr = self._kanadict.lookup(text[:x])
        return Hstr, max_len

    def convert_h(self, text):
        Hstr = ""
        max_len = 0
        r = len(text)
        for x in xrange(r):
            if self.isRegion(text[x]) and ord(text[x]) < 0x30f7:
                Hstr = Hstr + unichr(ord(text[x]) - self._diff)
                max_len += 1
            elif self.isRegion(text[x]):
                Hstr = Hstr + text[x]
                max_len += 1
            else:  # pragma: no cover
                break
        return (Hstr, max_len)

    def convert_noop(self, text):
        return text[0], 1


class Jisyo:
    _dict = None

    def __init__(self, dictname):
        self._dict = file_archive(Configurations().dictpath(dictname), {}, serialized=True)
        self._dict.load()

    def haskey(self, key):
        return key in self._dict

    def lookup(self, key):
        return self._dict[key]

    def maxkeylen(self):
        return self._dict['_max_key_len_']


class Sym2:

    def __init__(self, mode):
        if mode == "a":
            self.convert = self.convert_a
        else:
            self.convert = self.convert_noop

    def isRegion(self, char):
        c = ord(char[0])
        return (Ch.ideographic_space <= c <= Ch.postal_mark_face) or \
               (Ch.wavy_dash <= c <= Ch.ideographic_half_fill_space) or \
               (Ch.greece_Alpha <= c <= Ch.greece_Rho) or (Ch.greece_Sigma <= c <= Ch.greece_Omega) or \
               (Ch.greece_alpha <= c <= Ch.greece_omega) or \
               (Ch.cyrillic_A <= c <= Ch.cyrillic_ya) or \
               (Ch.zenkaku_exc_mark <= c <= Ch.zenkaku_number_nine) or \
               (0xff20 <= c <= 0xff5e) or c == 0x0451 or c == 0x0401

    def _convert(self, text):
        c = ord(text[0])
        if Ch.ideographic_space <= c <= Ch.postal_mark_face:
            return Convert_Tables.symbol_table_1[c - Ch.ideographic_space]
        elif Ch.wavy_dash <= c <= Ch.ideographic_half_fill_space:
            return Convert_Tables.symbol_table_2[c - Ch.wavy_dash]
        elif Ch.greece_Alpha <= c <= Ch.greece_Omega:
            return Convert_Tables.symbol_table_3[c - Ch.greece_Alpha]
        elif Ch.greece_alpha <= c <= Ch.greece_omega:
            return Convert_Tables.symbol_table_4[c - Ch.greece_alpha]
        elif Ch.cyrillic_A <= c <= Ch.cyrillic_ya:
            return Convert_Tables.cyrillic_table[text[0]]
        elif c == Ch.cyrillic_E or c == Ch.cyrillic_e:
            return Convert_Tables.cyrillic_table[text[0]]
        elif Ch.zenkaku_exc_mark <= c <= Ch.zenkaku_slash_mark:
            return Convert_Tables.symbol_table_5[c - Ch.zenkaku_exc_mark]
        elif Ch.zenkaku_number_zero <= c <= Ch.zenkaku_number_nine:
            return unichr(c - Ch.zenkaku_number_zero + ord('0'))
        elif 0xff20 <= c <= 0xff40:
            return unichr(0x0041 + c - 0xff21)  # u\ff21Ａ => u\0041:@A..Z[\]^_`
        elif 0xff41 <= c < 0xff5f:
            return unichr(0x0061 + c - 0xff41)  # u\ff41ａ => u\0061:a..z{|}
        else:
            return ""  # pragma: no cover

    def convert_a(self, text):
        t = self._convert(text)
        if len(t):
            return t, 1
        else:
            return "", 0

    def convert_noop(self, text):
        return text[0], 1


class A2:

    def __init__(self, mode):
        if mode == "E":
            self.convert = self.convert_E
        else:
            self.convert = self.convert_noop

    def isRegion(self, char):
        return Ch.space <= ord(char[0]) < Ch.delete

    def _convert(self, text):
        c = ord(text[0])
        if Ch.space <= c <= Ch.at_mark:
            return Convert_Tables.alpha_table_1[(c - Ch.space)]
        elif Ch.alphabet_A <= c <= Ch.alphabet_Z:
            return unichr(Ch.zenkaku_A + c - Ch.alphabet_A)  # u\0041A => u\ff21Ａ
        elif Ch.square_bra <= c <= Ch.back_quote:
            return Convert_Tables.alpha_table_2[(c - Ch.square_bra)]
        elif Ch.alphabet_a <= c <= Ch.alphabet_z:
            return unichr(Ch.zenkaku_a + c - Ch.alphabet_a)  # u\0061a => u\ff41ａ
        elif Ch.bracket_bra <= c <= Ch.tilda:
            return Convert_Tables.alpha_table_3[(c - Ch.bracket_bra)]
        else:
            return ""  # pragma: no cover

    def convert_E(self, text):
        t = self._convert(text)
        if len(t):
            return t, 1
        else:
            return "", 0

    def convert_noop(self, text):
        return text[0], 1



# -*- coding: utf-8 -*-

import threading

import pykakasi
import pykakasi.kanji
import pytest

I_TEST = [
    (u"菟", u"兎"),
    (u"菟集", u"兎集"),
    (u"熙", u"煕"),
    (u"壱弍", u"一二"),
    (u"森鷗外", u"森鴎外"),
]


@pytest.mark.thread
def test_thread_itaiji():
    for i in range(10):
        t = threading.Thread(target=worker_itaiji)
        t.start()
    main_thread = threading.currentThread()
    for t in threading.enumerate():
        if t is not main_thread:
            t.join()


def worker_itaiji():
    j = pykakasi.J2("H")
    for case, result in I_TEST:
        assert j.itaiji_conv(case) == result


@pytest.mark.thread
def test_thread_kanwa():
    for i in range(10):
        t = threading.Thread(target=worker_kanwa)
        t.start()
    main_thread = threading.currentThread()
    for t in threading.enumerate():
        if t is not main_thread:
            t.join()


def worker_kanwa():
    k = pykakasi.kanwa.kanwa()
    d = k.load(u"春")
    assert d[u"春"] is not None



# -*- coding: utf-8 -*-

import six

import pykakasi


def test_issues60():
    TESTS = [
        (u"市立", u"しりつ")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "H")
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_issues59():
    TESTS = [
        (u"じゃーん", u"じゃーん"),
        (u"ヷヸヹヺ", u"ヷヸヹヺ")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", "H")
    kakasi.setMode("J", None)
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_issue66():
    TESTS = [
        (u"月々", "tukizuki"),
        (u"毎月々", "maitukizuki"),
        (u"佐々木", "sasaki"),
        (u"中佐々木", "nakasasaki"),
        (u"代々木", "yoyogi"),
        (u"次代々木", "tugiyoyogi")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Kunrei")
    conv = kakasi.getConverter()
    for case, result in TESTS:
        assert conv.do(case) == result


def test_issues68():
    TESTS = [
        (u"", u""),
        (u"埇", u"よう")
    ]
    kks = pykakasi.kakasi()
    kks.setMode("J", "H")
    convert = kks.getConverter()
    for case, result in TESTS:
        assert convert.do(case) == result


def test_issue68_2():
    kks = pykakasi.kakasi()
    kks.setMode("J", "H")
    convert = kks.getConverter()
    for case in range(0x3400, 0xdfff):
        assert convert.do(six.unichr(case)) is not None
    for case in range(0xf900, 0xfa2e):
        assert convert.do(six.unichr(case)) is not None


def test_issue72():
    TESTS = [
        (u"㐂", u"き")
    ]
    kks = pykakasi.kakasi()
    kks.setMode("J", "H")
    convert = kks.getConverter()
    for case, result in TESTS:
        assert convert.do(case) == result


def test_issue78():
    TESTS = [
        (u"由来し、この", u"ゆらい し、 この"),
        (u"これは人類普遍であり、かかる原理に",
         u"これは じんるいふへん であり、 かかる げんり に")
    ]
    kks = pykakasi.kakasi()
    kks.setMode("J", "H")
    kks.setMode("s", True)
    convert = kks.getConverter()
    for case, result in TESTS:
        assert convert.do(case) == result



#!/usr/bin/env python

import io
import os
import re
import sys

from setuptools import setup
from setuptools.command.build_py import build_py

package_name = "pykakasi"
root_dir = os.path.abspath(os.path.dirname(__file__))


class MyBuild(build_py):

    def run(self):
        build_py.run(self)
        sys.path.insert(1, os.path.join(root_dir, 'src'))
        from kakasidict import Genkanwadict
        if not self.dry_run:
            kanwa = Genkanwadict()
            dstdir = os.path.join(self.build_lib, 'pykakasi', 'data')
            kanwa.generate_dictionaries(dstdir)


def readme():
    with io.open(os.path.join(root_dir, 'README.rst'), mode="r", encoding="UTF-8") as f:
        return f.read()


with open(os.path.join(root_dir, 'src', package_name, '__init__.py')) as f:
    init_text = f.read()
    version = re.search(r'__version__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    license = re.search(r'__license__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author = re.search(r'__author__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author_email = re.search(r'__author_email__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    url = re.search(r'__url__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)

assert version
assert license
assert author
assert author_email
assert url


setup(name=package_name,
      version=version,
      description='Python implementation of kakasi - kana kanji simple inversion library',
      url=url,
      license=license,
      long_description=readme(),
      author=author,
      author_email=author_email,
      package_dir={'pykakasi': 'src/pykakasi'},
      packages=[package_name],
      provides=[package_name],
      scripts=["bin/kakasi"],
      include_package_data=True,
      package_data={'src/data': ['*.utf8']},
      tests_require=['pytest', 'coverage'],
      setup_requires=['six', 'klepto'],
      install_requires=['six', 'klepto'],
      extras_require={'dev': ['pytest']},
      cmdclass={'build_py': MyBuild},
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Software Development :: Libraries :: Python Modules']
      )



import os
import re

from klepto.archives import file_archive
from six import unichr

root_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


class Genkanwadict(object):
    records = {}

    def run(self, src, dst):
        with open(src, 'rb') as f:
            for line in f:
                self.parsekdict(line.decode("utf-8"))
            f.close()
        self.kanwaout(dst)

    # for itaiji and kana dict

    def mkdict(self, src, dst):
        max_key_len = 0
        dic = {}
        with open(src, "rb") as f:
            for line in f:
                line = line.decode("utf-8").strip()
                if line.startswith(';;'):  # skip comment
                    continue
                if re.match(r"^$", line):
                    continue
                try:
                    (v, k) = (re.sub(r'\\u([0-9a-fA-F]{4})',
                                     lambda x: unichr(int(x.group(1), 16)), line)).split(' ')
                    dic[k] = v
                    max_key_len = max(max_key_len, len(k))
                except ValueError:
                    raise Exception("Cannot process dictionary line: ", line)
        d = file_archive(dst, dic, serialized=True)
        d['_max_key_len_'] = max_key_len
        d.dump()

    # for kanwadict

    def parsekdict(self, line):
        line = line.strip()
        if line.startswith(';;'):  # skip comment
            return
        (yomi, kanji) = line.split(' ')
        if ord(yomi[-1:]) <= ord('z'):
            tail = yomi[-1:]
            yomi = yomi[:-1]
        else:
            tail = ''
        self.updaterec(kanji, yomi, tail)

    def updaterec(self, kanji, yomi, tail):
        key = "%04x" % ord(kanji[0])
        if key in self.records:
            if kanji in self.records[key]:
                rec = self.records[key][kanji]
                rec.append((yomi, tail))
                self.records[key].update({kanji: rec})
            else:
                self.records[key][kanji] = [(yomi, tail)]
        else:
            self.records[key] = {}
            self.records[key][kanji] = [(yomi, tail)]

    def kanwaout(self, out):
        dic = file_archive(out, self.records, serialized=True)
        dic.dump()

    def generate_dictionaries(self, dstdir):
        DICTS = [
            ('itaijidict.utf8', 'itaijidict3.db'),
            ('hepburndict.utf8', 'hepburndict3.db'),
            ('kunreidict.utf8', 'kunreidict3.db'),
            ('passportdict.utf8', 'passportdict3.db'),
            ('hepburnhira.utf8', 'hepburnhira3.db'),
            ('kunreihira.utf8', 'kunreihira3.db'),
            ('passporthira.utf8', 'passporthira3.db')
        ]
        srcdir = os.path.join(root_dir, 'src', 'data')
        if not os.path.exists(dstdir):
            os.makedirs(dstdir)
        for (src_f, pkl_f) in DICTS:
            src = os.path.join(srcdir, src_f)
            dst = os.path.join(dstdir, pkl_f)
            if (os.path.exists(dst)):
                os.unlink(dst)
            self.mkdict(src, dst)
        src = os.path.join(srcdir, 'kakasidict.utf8')
        dst = os.path.join(dstdir, 'kanwadict4.db')
        if (os.path.exists(dst)):
            os.unlink(dst)
        self.run(src, dst)



from .kakasi import kakasi, wakati
from .kanji import J2
from .properties import Configurations
from .scripts import A2, H2, K2, Sym2

__copyright__ = 'Copyright (C) 2010-2019 Hiroshi Miura'
__version__ = '1.1b2'
__license__ = 'GPLv3'
__author__ = 'Hiroshi Miura'
__author_email__ = 'miurahr@linux.com'
__url__ = 'http://github.com/miurahr/pykakasi'

__all__ = ["kakasi", "wakati", "exceptions", "J2", "K2", "H2", "Sym2", "A2", "Configurations"]



# -*- coding: utf-8 -*-

import pykakasi
import pytest


def test_kakasi_hepburn():
    TESTS = [
        (u"", ""),
        (u"構成", "Kousei"),
        (u"好き", "Suki"),
        (u"大きい", "Ookii"),
        (u"かんたん", "kantan"),
        (u"にゃ", "nya"),
        (u"っき", "kki"),
        (u"っふぁ", "ffa"),
        (u"キャ", "kya"),
        (u"キュ", "kyu"),
        (u"キョ", "kyo"),
        (u"漢字とひらがな交じり文", "Kanji tohiragana Majiri Bun"),
        (u"Alphabet 123 and 漢字", "Alphabet 123 and Kanji"),
        (u"日経新聞", "Nikkeishinbun"),
        (u"日本国民は、", "Nihonkokumin ha,")
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_kunrei():
    TESTS = [
        (u"", ""),
        (u"構成", "Kousei"),
        (u"好き", "Suki"),
        (u"大きい", "Ookii"),
        (u"かんたん", "kantan"),
        (u"にゃ", "nya"),
        (u"っき", "kki"),
        (u"っふぁ", "ffa"),
        (u"漢字とひらがな交じり文", "Kanzi tohiragana Maziri Bun"),
        (u"Alphabet 123 and 漢字", "Alphabet 123 and Kanzi"),
        (u"日経新聞", "Nikkeisinbun"),
        (u"日本国民は、", "Nihonkokumin ha,")
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Kunrei")
    kakasi.setMode("E", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("a", None)
    kakasi.setMode("C", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_J2H():

    TESTS = [
        (u"", ""),
        (u"構成", u"こうせい"),
        (u"好き", u"すき"),
        (u"大きい", u"おおきい"),
        (u"かんたん", u"かんたん"),
        (u"にゃ", u"にゃ"),
        (u"っき", u"っき"),
        (u"っふぁ", u"っふぁ"),
        (u"漢字とひらがな交じり文", u"かんじとひらがなまじりぶん"),
        (u"Alphabet 123 and 漢字", u"Alphabet 123 and かんじ"),
        (u"日経新聞", u"にっけいしんぶん"),
        (u"日本国民は、", u"にほんこくみんは、"),
        (u"苦々しい", u"にがにがしい")
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "H")
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_J2K():

    TESTS = [
        (u"", ""),
        (u"構成", u"コウセイ"),
        (u"好き", u"スキ"),
        (u"大きい", u"オオキイ"),
        (u"かんたん", u"かんたん"),
        (u"漢字とひらがな交じり文", u"カンジとひらがなマジリブン"),
        (u"Alphabet 123 and 漢字", u"Alphabet 123 and カンジ")
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "K")
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_H2K():

    TESTS = [
        (u"", ""),
        (u"かんたん", u"カンタン"),
        (u"にゃ", u"ニャ")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "K")
    kakasi.setMode("S", " ")
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_K2H():

    TESTS = [
        (u"", ""),
        (u"カンタン", u"かんたん"),
        (u"ニャ", u"にゃ")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("K", "H")
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_wakati():
    TESTS = [
        (u"", u""),
        (u"交じり文", u"交じり 文"),
        (u"ひらがな交じり文", u"ひらがな 交じり 文"),
        (u"漢字とひらがな交じり文", u"漢字 とひらがな 交じり 文")
    ]
    wakati = pykakasi.wakati()
    converter = wakati.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_katakana_furiagana():
    TESTS = [
        (u"変換前の漢字の脇に", u"変換前[ヘンカンマエ]の漢字[カンジ]の脇[ワキ]に")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "KF")
    kakasi.setMode("f", True)
    kakasi.setMode("s", False)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_hiragana_furiagana():
    TESTS = [
        (u"変換前の漢字の脇に", u"変換前[へんかんまえ]の漢字[かんじ]の脇[わき]に")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "HF")
    kakasi.setMode("f", True)
    kakasi.setMode("s", False)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


@pytest.mark.xfail(reason="Not implemented yet furigana mode for wakati")
def test_wakati_furiagana():
    TESTS = [
        (u"変換前の漢字の脇に", u"変換前[へんかんまえ] の 漢字[かんじ] の 脇[わき] に")
    ]
    kakasi = pykakasi.wakati()
    kakasi.setMode("f", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_a2E():
    TESTS = [
        ("ABCDEFGHIJKLMNOPQRSTUVWXYZ", u"ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ"),
        ("abcdefghijklmnopqrstuvwxyz", u"ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ"),
        ("!\"#$%&'()*+,-./_ {|}~", u"！＂＃＄％＆＇（）＊＋，－．／＿　｛｜｝～")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("a", "E")
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_E2a():

    TESTS = [
        (u"ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"),
        (u"ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ", "abcdefghijklmnopqrstuvwxyz"),
        (u"！＂＃＄％＆＇（）＊＋，－．／＿　｛｜｝～\uFF1A", "!\"#$%&'()*+,-./_ {|}~:")
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode("E", "a")
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_E2a_upper():
    TESTS = [
        (u"ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"),
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("E", "a")
    kakasi.setMode("U", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_J2a_upper():
    TESTS = [
        (u"かな漢字", "kana KANJI"),
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("J", "a")
    kakasi.setMode("H", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("U", True)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_numbers():

    TESTS = [
        (u"１２３４５６７８９０", "1234567890"),
        (u"一 二 三 四 五 六 七 八 九 〇", "ichi ni san shi go roku shichi hachi kyuu (maru)")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("E", "a")
    kakasi.setMode("J", "a")
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_passport():

    TESTS = [
        (u"", ""),
        (u"構成", "Kosei"),
        (u"大野", "Ono"),
        (u"斎藤", "Saito"),
        (u"菅野", "Kanno"),
        (u"本田", "Honda"),
        (u"一式", "Isshiki"),
        (u"別府", "Beppu"),
        (u"ジェ", "jie"),
        (u"チェ", "chie"),
        (u"ティ", "tei"),
        (u"ディ", "dei"),
        (u"デュ", "deyu"),
        (u"ファ", "fua"),
        (u"フィ", "fui"),
        (u"フェ", "fue"),
        (u"フォ", "fuo"),
        (u"ヴァ", "bua"),
        (u"ヴィ", "bui"),
        (u"ヴ", "bu"),
        (u"ヴェ", "bue"),
        (u"ヴォ", "buo"),
        (u"キャ", "kya"),
        (u"キュ", "kyu"),
        (u"キョ", "kyo"),
        (u"じぇ", "jie"),
        (u"ちぇ", "chie"),
        (u"てぃ", "tei"),
        (u"でぃ", "dei"),
        (u"でゅ", "deyu"),
        (u"ふぁ", "fua"),
        (u"ふぃ", "fui"),
        (u"ふぇ", "fue"),
        (u"ふぉ", "fuo")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Passport")
    kakasi.setMode("E", "a")
    kakasi.setMode("C", True)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_passport_specialcase():

    TESTS = [
        (u"えっちゅう", "etchu"),
        (u"はっちょう", "hatcho"),
        (u"エッチュウ", "etchu"),
        (u"ハッチョウ", "hatcho")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Passport")
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_hepburn_nocapital():

    TESTS = [
        (u"", ""),
        (u"構成", "kousei"),
        (u"好き", "suki"),
        (u"大きい", "ookii"),
        (u"かんたん", "kantan"),
        (u"にゃ", "nya"),
        (u"っき", "kki"),
        (u"っふぁ", "ffa"),
        (u"漢字とひらがな交じり文", "kanji tohiragana majiri bun"),
        (u"Alphabet 123 and 漢字", "Alphabet 123 and kanji"),
        (u"日経新聞", "nikkeishinbun"),
        (u"日本国民は、", "nihonkokumin ha,")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


@pytest.mark.xfail(reason="Cannot handle small kana extension")
def test_kakasi_extended_kana():
    TESTS = [
        (u"\U0001b150", "wi"),
        (u"\U0001b151", "we")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_chinese_kanji():
    TESTS = [
        (u"您好", u'您 kou')
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_chinese_kanji_replace():
    TESTS = [
        (u"您好", u'??? kou')
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("s", True)
    kakasi.setMode("E", "a")
    kakasi.setMode("a", None)
    kakasi.setMode("C", False)
    kakasi.setMode("t", False)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_long_symbol_H():
    TESTS = [
        (u'やったー', u'yattaa'),
        (u'でっでー', u'deddee'),
        (u'てんさーふろー', u'tensaafuroo'),
        (u'がっがーん', u'gaggaan'),
        (u'どーん', u'doon')
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode('H', 'a')
    converter = kakasi.getConverter()
    for case, result in TESTS:
        print(converter.do(case))
        assert converter.do(case) == result


def test_kakasi_long_symbol_K():
    TESTS = [
        (u'ヤッター', u'yattaa'),
        (u'デッデー', u'deddee'),
        (u'テンサーフロー', u'tensaafuroo'),
        (u'ガッガーン', u'gaggaan'),
        (u'ドーン', u'doon')
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    converter = kakasi.getConverter()
    for case, result in TESTS:
        print(converter.do(case))
        assert converter.do(case) == result


def test_kakasi_long_symbol_mixed_HK():
    TESTS = [
        (u'じゃーんデデーン', u'jaandedeen'),
        (u'デッデーンじゃーん', u'deddeenjaan'),
        (u'テンサーふろー', u'tensaafuroo'),
        (u'ガッガーン', u'gaggaan'),
        (u'ドーン', u'doon')
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    kakasi.setMode('H', 'a')
    converter = kakasi.getConverter()
    for case, result in TESTS:
        print(converter.do(case))
        assert converter.do(case) == result


def test_kakasi_long_symbol_mixed_JHK():
    TESTS = [
        (u'順じゃーんデデーン', u'junjaandedeen'),
        (u'デッデーン順じゃーん', u'deddeenjunjaan'),
        (u'テンサーふろー風呂', u'tensaafuroofuro'),
        (u'ガッガーン癌', u'gaggaangan'),
        (u'ドーン', u'doon')
    ]

    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    kakasi.setMode('H', 'a')
    kakasi.setMode('J', 'a')
    converter = kakasi.getConverter()
    for case, result in TESTS:
        print(converter.do(case))
        assert converter.do(case) == result


@pytest.mark.skip
def test_kakasi_long_symbol_with_no_HK():
    TESTS = [
        (u'順ーデデーン', u'jun-dedeen'),
        (u'デッデーン順ー', u'deddeenjun-')
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode('K', 'a')
    kakasi.setMode('H', 'a')
    kakasi.setMode('J', 'a')
    converter = kakasi.getConverter()
    for case, result in TESTS:
        print(converter.do(case))
        assert converter.do(case) == result
